/****************************************************************************
** Meta object code from reading C++ file 'konfigurator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kwalletmanager-17.08.3/src/konfigurator/konfigurator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'konfigurator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KWalletConfig_t {
    QByteArrayData data[11];
    char stringdata0[146];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KWalletConfig_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KWalletConfig_t qt_meta_stringdata_KWalletConfig = {
    {
QT_MOC_LITERAL(0, 0, 13), // "KWalletConfig"
QT_MOC_LITERAL(1, 14, 13), // "configChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 13), // "launchManager"
QT_MOC_LITERAL(4, 43, 14), // "newLocalWallet"
QT_MOC_LITERAL(5, 58, 16), // "newNetworkWallet"
QT_MOC_LITERAL(6, 75, 17), // "updateWalletLists"
QT_MOC_LITERAL(7, 93, 9), // "newWallet"
QT_MOC_LITERAL(8, 103, 11), // "deleteEntry"
QT_MOC_LITERAL(9, 115, 26), // "customContextMenuRequested"
QT_MOC_LITERAL(10, 142, 3) // "pos"

    },
    "KWalletConfig\0configChanged\0\0launchManager\0"
    "newLocalWallet\0newNetworkWallet\0"
    "updateWalletLists\0newWallet\0deleteEntry\0"
    "customContextMenuRequested\0pos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KWalletConfig[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x0a /* Public */,
       3,    0,   55,    2, 0x0a /* Public */,
       4,    0,   56,    2, 0x0a /* Public */,
       5,    0,   57,    2, 0x0a /* Public */,
       6,    0,   58,    2, 0x0a /* Public */,
       7,    0,   59,    2, 0x0a /* Public */,
       8,    0,   60,    2, 0x0a /* Public */,
       9,    1,   61,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   10,

       0        // eod
};

void KWalletConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KWalletConfig *_t = static_cast<KWalletConfig *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->configChanged(); break;
        case 1: _t->launchManager(); break;
        case 2: _t->newLocalWallet(); break;
        case 3: _t->newNetworkWallet(); break;
        case 4: _t->updateWalletLists(); break;
        case 5: { QString _r = _t->newWallet();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->deleteEntry(); break;
        case 7: _t->customContextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject KWalletConfig::staticMetaObject = {
    { &KCModule::staticMetaObject, qt_meta_stringdata_KWalletConfig.data,
      qt_meta_data_KWalletConfig,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *KWalletConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWalletConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KWalletConfig.stringdata0))
        return static_cast<void*>(this);
    return KCModule::qt_metacast(_clname);
}

int KWalletConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KCModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
